function zdot = cstr_calc_pp(t, z, theta, mode, headspace, temp, mixing, time, o2mode, flow)


% Dissolved o2 calculations
kh = 0.0013; % mol/kg*bar
dh = 1500; % 1/k
H2O_mm = 18.01528; %g/mol;
O2_mm = 31.9980; % O2 molar mass in g/mol
CO2_mm = 44.01; %CO2 molar mass in g/mol
O2_rho = 1.43; % O2 densitity at stp
ReactorVol = 300;

numruns = length(z)/14;


%Data
pCOD_f = z(1:14:end);
pCOD_s = z(2:14:end);
% pCOD = z(3:14:end);
sCOD_f = z(4:14:end);
sCOD_s = z(5:14:end);
% sCOD_nr = z(6:14:end);
% sCOD = z(7:14:end);
% AACOD = z(8:14:end);
% CO2 = z(9:14:end);
O2gas = z(10:14:end);
DON = z(11:14:end);
VFACOD_f = z(12:14:end);
VFACOD_s = z(13:14:end);
% VFACOD = z(14:14:end);  


%temperature
switch(mode)
    case 'data'
        temperature = interp1(time, temp, t)';
    case 'deriv'
        %temperature = temp{i+1}.a*exp(temp{i+1}.b*t) + temp{i+1}.c*exp(temp{i+1}.d*t);
        temp = [temp{:}];
        temperature = ([temp.a1].*exp(-((t-[temp.b1])./[temp.c1]).^2) + [temp.a2].*exp(-((t-[temp.b2])./[temp.c2]).^2) + [temp.a3].*exp(-((t-[temp.b3])./[temp.c3]).^2))';
end

if(~isa(t,'barvec'))
    zdot = zeros(numruns,14);
end
    AR = [theta(1) theta(2) theta(3) theta(4) theta(5) theta(6) theta(7) theta(8) theta(9) theta(10) theta(11) theta(12)];
    O2order = [theta(13) theta(14) theta(15) theta(16) theta(17) theta(18) theta(19) theta(20) theta(21) theta(22) theta(23) theta(24)];
    EA = [theta(25) theta(26) theta(27) theta(28) theta(29) theta(30) theta(31) theta(32) theta(33) theta(34) theta(35) theta(36)];
    Mix = [theta(37) theta(38) theta(39) theta(40) theta(41) theta(42) theta(43) theta(44) theta(45) theta(46) theta(47) theta(48)];
    
    AR = AR*1e3;
    AR(5) = AR(5)*1e5;
    EA = EA*1e5;
    
    % sCOD_frac = [0.6 0.3 0.1];
    sCOD_frac = [theta(49) theta(50) 1-theta(49)-theta(50)];    
    vfa_frac = [theta(51) 1-theta(51)];
    
% Fit Temperature and Gas Volume Functions
switch(mode)
    case 'data'
        gas_volume = (interp1(time, headspace, t)');
        %gas_volume = headspace(1)/1000;
    case 'deriv'
        gas_volume = headspace.a*exp(headspace.b*t) + headspace.c*exp(headspace.d*t);
end    

if(o2mode==1)    
        O2gas = interp1(time, flow, t);
end
    %gas = O2gas*gas_volume;  
    p = ((O2gas./1000./O2_mm).*8.314.*temperature)./(gas_volume./1e3)./1e5; % partial pressure after heating (bar)

    o2_dissolved = kh*exp(dh*((1./temperature)-(1/298))); % o2 dissolved in water mol/kg*bar
    o2_dissolvedmgL = o2_dissolved.*H2O_mm.*(p).*1000; % mg/L

    
    
    o2_flow = interp1(time, flow, t); %(L)
    o2_density = (p*0.986923267*O2_mm)/(0.08206*temperature); %(g/L)
    o2_flow_mass  = o2_flow*o2_density*1000; %(mg)

    % think I'll have to change to a concentration basis for oxygen
    
    
    %Reaction where k represents Arr + mixing
    %rxn = @(c, o, oOrder, k) (c*(o^oOrder)*k)/1e3;
    
    rxnAR = @(c, o, oOrder, k, Ea,mix) (c.*(o.^oOrder).*k.*exp(-Ea./(8.3145*temperature)).*mixing.^mix)*1e-3; % units mgCOD/l s

    % Reaction Differential Equations  (units mgCOD/l except for O2gas which is in mg)
    %pCOD_f
    zdot(:,1) = -rxnAR(pCOD_f, o2_dissolvedmgL, O2order(1), AR(1), EA(1), Mix(1));% - rxnAR(pCOD_f, o2_dissolvedmgL, O2order(7), AR(7), EA(7), Mix(7)); 
    %pCOD_s
    zdot(:,2) = -rxnAR(pCOD_s, o2_dissolvedmgL, O2order(5), AR(5), EA(5), Mix(5)) - rxnAR(pCOD_s, o2_dissolvedmgL, O2order(7), AR(7), EA(7), Mix(7));  
    %pCOD
    zdot(:,3) = zdot(:,1)+zdot(:,2);
    %sCOD_f
    zdot(:,4) = sCOD_frac(1)*rxnAR(pCOD_f, o2_dissolvedmgL, O2order(1), AR(1), EA(1), Mix(1)) + sCOD_frac(1)*rxnAR(pCOD_s, o2_dissolvedmgL, O2order(5), AR(5), EA(5), Mix(5)) - rxnAR(sCOD_f, o2_dissolvedmgL, O2order(2), AR(2), EA(2), Mix(2)) - rxnAR(sCOD_f, o2_dissolvedmgL, O2order(3), AR(3), EA(3), Mix(3))  - rxnAR(sCOD_f, o2_dissolvedmgL, O2order(8), AR(8), EA(8), Mix(8)) ; 
    %sCOD_s
    zdot(:,5) = sCOD_frac(2)*rxnAR(pCOD_f, o2_dissolvedmgL, O2order(1), AR(1), EA(1), Mix(1)) + sCOD_frac(2)*rxnAR(pCOD_s, o2_dissolvedmgL, O2order(5), AR(5), EA(5), Mix(5)) - rxnAR(sCOD_s, o2_dissolvedmgL, O2order(6), AR(6), EA(6), Mix(6)) - rxnAR(sCOD_s, o2_dissolvedmgL, O2order(4), AR(4), EA(4), Mix(4)) - rxnAR(sCOD_s, o2_dissolvedmgL, O2order(9), AR(9), EA(9), Mix(9)) ; 
    %sCODnr
    zdot(:,6) = sCOD_frac(3)*rxnAR(pCOD_f, o2_dissolvedmgL, O2order(1), AR(1), EA(1), Mix(1)) + sCOD_frac(3)*rxnAR(pCOD_s, o2_dissolvedmgL, O2order(5), AR(5), EA(5), Mix(5)); 
    %sCOD
    zdot(:,7) = zdot(:,4)+zdot(:,5)+zdot(:,6);
    %AACOD
    zdot(:,8) = rxnAR(sCOD_f, o2_dissolvedmgL, O2order(2), AR(2), EA(2), Mix(2)) + rxnAR(sCOD_s, o2_dissolvedmgL, O2order(6), AR(6), EA(6), Mix(6)); 
    %CO2 (in mg)  
    zdot(:,9) =  (0*rxnAR(sCOD_f, o2_dissolvedmgL, O2order(3), AR(3), EA(3), Mix(3)) + 0.1*rxnAR(sCOD_s, o2_dissolvedmgL, O2order(4), AR(4), EA(4), Mix(4)) + 0.8*rxnAR(VFACOD_f, o2_dissolvedmgL, O2order(10), AR(10), EA(10), Mix(10))  + 1*rxnAR(VFACOD_s, o2_dissolvedmgL, O2order(12), AR(12), EA(12), Mix(12) + 1.5*rxnAR(DON, o2_dissolvedmgL, O2order(11), AR(11), EA(11), Mix(11)))*((ReactorVol-gas_volume)))*(CO2_mm/O2_mm); %+ rxnAR(DON, o2_dissolvedmgL, O2order(11), AR(11), EA(11), Mix(11))
    %O2gas ( mg)
    zdot(:,10) = o2_flow_mass - (((rxnAR(sCOD_f, o2_dissolvedmgL, O2order(2), AR(2), EA(2), Mix(2)) + rxnAR(sCOD_f, o2_dissolvedmgL, O2order(3), AR(3), EA(3), Mix(3)) + rxnAR(sCOD_s, o2_dissolvedmgL, O2order(4), AR(4), EA(4), Mix(4)) + rxnAR(sCOD_s, o2_dissolvedmgL, O2order(6), AR(6), EA(6), Mix(6))  + rxnAR(sCOD_f, o2_dissolvedmgL, O2order(8), AR(8), EA(8), Mix(8)) + rxnAR(sCOD_s, o2_dissolvedmgL, O2order(9), AR(9), EA(9), Mix(9)) + rxnAR(VFACOD_f, o2_dissolvedmgL, O2order(10), AR(10), EA(10), Mix(10)) + rxnAR(DON, o2_dissolvedmgL, O2order(11), AR(11), EA(11), Mix(11)) + rxnAR(VFACOD_s, o2_dissolvedmgL, O2order(12), AR(12), EA(12), Mix(12)))))*(ReactorVol-gas_volume); % *(ReactorVol-gas_volume) added change in volume due to sampling (was 400)

    %DON 
    zdot(:,11) = rxnAR(pCOD_s, o2_dissolvedmgL, O2order(7), AR(7), EA(7), Mix(7)) - rxnAR(DON, o2_dissolvedmgL, O2order(11), AR(11), EA(11), Mix(11));
    %VFACOD
    zdot(:,12) = vfa_frac(1)*rxnAR(sCOD_f, o2_dissolvedmgL, O2order(8), AR(8), EA(8), Mix(8)) + vfa_frac(1)*rxnAR(sCOD_s, o2_dissolvedmgL, O2order(9), AR(9), EA(9), Mix(9)) - rxnAR(VFACOD_f, o2_dissolvedmgL, O2order(10), AR(10), EA(10), Mix(10)) ;
    zdot(:,13) = vfa_frac(2)*rxnAR(sCOD_f, o2_dissolvedmgL, O2order(8), AR(8), EA(8), Mix(8)) + vfa_frac(2)*rxnAR(sCOD_s, o2_dissolvedmgL, O2order(9), AR(9), EA(9), Mix(9)) - rxnAR(VFACOD_s, o2_dissolvedmgL, O2order(12), AR(12), EA(12), Mix(12));
    zdot(:,14) = zdot(:,12) + zdot(:,13);
%     zdot(:,i+1) = zl;
% end

zdot = zdot';
zdot = zdot(:);
