clc
clear all
format longG

%Load Scion Measurements
load SCION_DATA;

% Constants
states = 14; %number of model states (used for indexing)
CO2 = 0; %Guesses Of Initial Concentration CO2
O2_mm = 31.9980; %O2 molar mass in g/mol
CO2_mm = 44.01; %CO2 molar mass in g/mol
H2O_mm = 18.01528; %H2O molar mass in g/mol
ReactorVolume = 600; %Reactor volume in ml
Water = 250; %Water added to reactor (ml)
fillTemperature = 293.15; %Fill temperature (K)

% Dissolved o2 calculation henry coefficients 
kh = 0.0013; % mol/kg*bar
d = 1500; % 1/k

% Initialise variables
x = 1; % counter for indexing
fitydata = [];
xt = (0:10:3600)';
temp = cell(19,1);

%Runs of Interest
runs = 1:19;

%% for image analysis and justification plots

% runs = 7:7;
% runs = 2:2;

%% Image analysis setup, manually modify an existing run (run 7)

run(7).o2 = 20;
run(7).temp = 220;

%% Justifying the model setup, manually modify run 2 to do the change in O2 conc
run(2).o2 = 60;
run(2).temp = 220;

%%

for r=runs(1):runs(end)    
    % Inital conditions
    Sludge = run(r).sludge;
    TotLiquid = Water+Sludge;
    GasBeforeSludge = ReactorVolume-Water;
    GasAfterSludge = ReactorVolume-(Water+Sludge); %Gas vol after water and sludge
    
    vapPress = (Props('P','T',fillTemperature,'Q',0,'Water')/100)*1e5; %in Pa
    
    if run(r).noflush == 1
        % no oxygen purge so calculate amount of nitrogen in reactor        
        n2Mole = ((101e3-vapPress)*(run(r).water/1e6))/(8.3145*fillTemperature); % reactor headspace flushed with pure nitrogen before oxygen addition
        o2Mole = (((run(r).o2*1e5)+101e3-vapPress)*(ReactorVolume-run(r).water)/1e6)/(8.3145*fillTemperature);
        %o2Mole = ((run(r).o2*1e5)*(ReactorVolume-run(r).water)/1e6)/(8.3145*(fillTemperature+273.15));
        o2Mole = o2Mole - n2Mole;
        
        fillPressure = (((o2Mole-n2Mole)*8.3145*fillTemperature)/((ReactorVolume-run(r).water)/1e6))/1e5;        
    else
        %Initial Concentration of O2 with oxygen purge (no nitrogen in reactor)
        o2Mole = (((run(r).o2*1e5)+101e3-vapPress)*((ReactorVolume-run(r).water)/1e6))/(8.3145*fillTemperature); % calculate moles of O2 in gas using ideal gas law
        fillPressure = ((run(r).o2*1e5)+101e3)/1e5;
    end
    
    
    %Initial Concentration of O2 in gas phase at room temperature  
    
    o2Mass = o2Mole*O2_mm; %mass of o2 in gas phase (g)
    
    % Calculate dissolved o2 at room temperature    
    o2_dissolved = kh*exp(d*((1./fillTemperature)-(1/298))); % o2 dissolved in water (mol/kg*bar)
    o2_dissolvedmgL = o2_dissolved.*H2O_mm.*(fillPressure).*1000; % (mg/L)
    
    % Total oxygen in reactor    
    massO2Liquid = o2_dissolvedmgL*Water/1e3; % (mg)
    totalO2InReactor = (massO2Liquid/1000)+o2Mass; %mass in (g)
    
    % Oxygen dissolved at operating temperature    
    p = ((totalO2InReactor/O2_mm)*8.314*(run(r).temp(1)+273.15))/(GasAfterSludge/1e6)/1e5; % partial pressure after heating (bar)
    o2_dissolved_t0 = kh*exp(d*((1./(run(r).temp(1)+273.15))-(1/298))); % o2 dissolved in water (mol/kg*bar)
    o2_dissolvedmgL_t0 = o2_dissolved_t0.*H2O_mm.*(p).*1000; % (mg/L)
    
    % Total oxygen at operating temperature after sludge added    
    massO2Liquid_t0 = o2_dissolvedmgL_t0*(TotLiquid/1e3); % (mg)
    totalO2InGas_t0 = (totalO2InReactor - massO2Liquid_t0/1000); %o2 in headspace (g)
    
    % Build reactor liquid level profile (due to sampling)    
    gas_volume = ones(length(run_lb(r).pressure),1)*GasAfterSludge;
    gas_volume(12:30) = 220;
    gas_volume(30:60) = 240;
    gas_volume(60:120) = 280;
    gas_volume(120:end) = 300;
    
    liq_p = ((totalO2InReactor/O2_mm)*8.314*(run_lb(r).temp+273.15))./(gas_volume/1e6)/1e5;
    
    %Form Initial State Vector pCOD_f pCOD_s pCOD sCOD_f sCOD_s sCOD_nr sCOD AA nonAA VFA CO CO2 O2liq O2gas
    % original fractions 0.82 0.18 0.6 0.3 0.1

    z_prev(:,x) = [ run(r).pcod(1)*0.84
        run(r).pcod(1)*0.16
        run(r).pcod(1)
        (run(r).scod(1)-run(r).vfa(1)).*0.3
        (run(r).scod(1)-run(r).vfa(1))*0.5
        (run(r).scod(1)-run(r).vfa(1))*0.2
        run(r).scod(1)-run(r).vfa(1)
        run(r).ac(1)
        CO2
        totalO2InGas_t0*1000
        run(r).don(1)
        (run(r).vfa(1)-run(r).ac(1))*0.7
        (run(r).vfa(1)-run(r).ac(1))*0.3
        run(r).vfa(1)-run(r).ac(1)];      
    
     z(:,x) = [ run(r).pcod(1)
        run(r).pcod(1)
        run(r).pcod(1)
        (run(r).scod(1)-run(r).vfa(1)).*0.3
        (run(r).scod(1)-run(r).vfa(1))*0.5
        (run(r).scod(1)-run(r).vfa(1))*0.2
        run(r).scod(1)-run(r).vfa(1)
        run(r).ac(1)
        CO2
        totalO2InGas_t0*1000
        run(r).don(1)
        (run(r).vfa(1)-run(r).ac(1))*0.7
        (run(r).vfa(1)-run(r).ac(1))*0.3
        run(r).vfa(1)-run(r).ac(1)];   
    
    fitxdata(:,x) = run(r).time;
    
    if run(r).O2gasfrac(end) > 0 % if run has gas data
        
        % prepend calculated inital conditions for o2 and co2 as we don't have
        % a t0 gas sample.
        run(r).CO2gasfrac(1) = 0;
        run(r).O2gasfrac(1) = 1;
        
        run(r).O2gasmass = run(r).O2gasfrac*totalO2InGas_t0;
        run(r).CO2gasmass = run(r).CO2gasfrac*totalO2InGas_t0*(CO2_mm/O2_mm);     
        
        %fitydata = [fitydata [run(r).pcod run(r).scod-run(r).vfa run(r).ac run(r).CO2gasmass*1000 run(r).O2gasfrac*totalO2InGas_t0*1000 run(r).don run(r).vfa-run(r).ac]];
        fitydata = [fitydata [run(r).pcod run(r).scod-run(r).vfa run(r).ac run(r).O2gasfrac*totalO2InGas_t0*1000 run(r).don run(r).vfa-run(r).ac]];

    else
        run(r).O2gasmass = zeros(length(run(r).O2gasfrac),1);
        run(r).CO2gasmass = zeros(length(run(r).O2gasfrac),1);
        
        fitydata = [fitydata [run(r).pcod run(r).scod-run(r).vfa run(r).ac run(r).don run(r).vfa-run(r).ac]];
    end
    
    x = x + 1;
end
z_prev = z_prev(:);
z0 = z(:);

%%
%Create parameter vector (theta)

ar=[1.8
          0.79485
           0.92597
         350.574026139142129 
          30.2656022084079
         0.293488545883267
        0.594661484838351 
               0.1526333 %0.002526333
        0.0134430715091141
         0.869170145716102
        0.0114301957343009
         0.095928982356516]';


o2order = [         0.299605073861853
         0.886589177295454
         0.219100051097171
         0.925864750186 
         0.440205159262489
         0.248547314468428
         0.553828005481191
               -1.8736
         0.301627455525776
         0.364097328818647
         0.786130663591281
         0.40]';


arEa = [         0.401395704002056
         0.599175067950098
          0.35484705597372
         1.055676084595672 
          1.24094369530074
         0.472359392180145
         0.483266965035988
               0.788365906 %1.788365906
         0.369764113444974
         0.345893785467609
         0.304417174544645
         0.43
]';

mixingorder = [         0.472071394186516
         0.545636358118566
         0.468192344739728
      1.2
         0.742152300876037
         0.455078638203035
        0.0534751244544175
                    0.0238
         0.558189360812938
        0.0624943390960389
      0.1 %  0.001
         0.36
]';



theta = [ar o2order arEa mixingorder 0.5 0.3 0.5 0.84]; % added pCODf frac

xt = (0:10:3600)';
gf = fit(xt,gas_volume,'exp2');

%ODE Function for Integrator
ode = @(t,z) cstr_calc(t,z,theta,'data',gas_volume,[run_lb(runs).temp],[run(runs).mixing]');
% ode = @(t,z) cstr_calc(t,z,theta,'data',run_lb(r).temp,gas_volume);
tic
%Solve System
[t,y] = ode45(ode,[0 3600],z_prev);
toc

%Run Simulation and Plot Results
cstr_vec_plot(run,t,y,1:19);
squeezefigs(3,3)

%%
cstr_vec_plot(run,t,y,2);
squeezefigs(3,3)


%%

% Normalise 
xdata = run(r).time;

pcod = [run(runs).pcod];
pcod = pcod(:);
pcod_min = min(pcod);
pcod_max = max(pcod);

scod = [run(runs).scod]-[run(runs).vfa];
scod = scod(:);
scod_min = min(scod);
scod_max = max(scod);

ac = [run(runs).ac];
ac = ac(:);
ac_min = min(ac);
ac_max = max(ac);

vfa = [run(runs).vfa]-[run(runs).ac];
vfa = vfa(:);
vfa_min = min(vfa);
vfa_max = max(vfa);

o2 = [run(runs).O2gasmass];
o2 = o2(:);
o2_min = min(o2);
o2_max = max(o2);

don = [run(runs).don];
don = don(:);
don_min = min(don);
don_max = max(don);

pcod_min = ones(length(xdata),1)*pcod_min;
scod_min = ones(length(xdata),1)*scod_min;
ac_min = ones(length(xdata),1)*ac_min;
vfa_min = ones(length(xdata),1)*vfa_min;
o2_min = ones(length(xdata),1)*o2_min;
don_min = ones(length(xdata),1)*don_min;

pcod_range = ones(length(xdata),1)*(pcod_max-pcod_min(1));
scod_range = ones(length(xdata),1)*(scod_max-scod_min(1));
ac_range = ones(length(xdata),1)*(ac_max-ac_min(1));
vfa_range = ones(length(xdata),1)*(vfa_max-vfa_min(1));
o2_range = ones(length(xdata),1)*(o2_max-o2_min(1))*1000;
don_range = ones(length(xdata),1)*(don_max-don_min(1));

%% Optimise parameters
nparam = length(theta);
%Bounds
lb = [1e-6*ones(nparam-1,1); 0.6];
ub = [5; ones(2,1)*3.2; 400; 40; ones(7,1)*2; ones(12,1)*1.5; ones(12,1)*3; ones(12,1)*1.5; 0.7; 0.3; 0.6; 0.9];%; 0.9; 0.9];

nz = length(z0);
nparam = length(theta);

fitxdata = fitxdata(:);  
xdata = run(r).time;

% Construct fitting data, fitting gas data when availabe
fitindex = [3 7 8 11 14];
fitindexes = []; 
wts = [];
z0index = [1];
z0indexes = [];
min = [pcod_min scod_min ac_min don_min vfa_min]; range = [pcod_range scod_range ac_range don_range vfa_range];
minidx = []; rangeidx = [];

for i=0:(nz/states)-1
        pfit = ones(size(fitydata,1),1);
        otherfit = pfit;
        otherfit(end,1) = pfit(end,1)*1.25;
        otherfit(4,1) = 1.25; % weight 4th point and end higher than the rest
        pfit(end,1) = pfit(end,1)*10;
        pfit(4,1) = 2;

    if run(runs(i+1)).O2gasfrac(end) > 0 % if run has gas data
        %fitindexes = [fitindexes ([3 7 8 9 10 11 14] + (i*states))];
        fitindexes = [fitindexes ([3 7 8 10 11 14] + (i*states))];        
        wts = [wts pfit repmat(otherfit,1,5)];         
        z0indexes = [z0indexes (z0index+(i*states))];
        
        minidx = [minidx ([pcod_min scod_min ac_min o2_min don_min vfa_min])]; rangeidx = [rangeidx ([pcod_range scod_range ac_range o2_range don_range vfa_range])];
    else
        fitindexes = [fitindexes (fitindex+(i*states))];
        %wts = [wts pfit pfit.*ones(size(fitydata,1),2) pfit.*1*ones(size(fitydata,1),1) ones(size(fitydata,1),1)];
        wts = [wts pfit repmat(otherfit,1,4)];
        z0indexes = [z0indexes (z0index+(i*states))];
        
        minidx = [minidx min]; rangeidx = [rangeidx range];
    end
end


theta = theta(:); % force to be a column 

% fun_swap = @(theta_optim) cstr_optim(flipud(theta_optim),'data',gas_volume,temp,run,run_lb,fitindexes,fitxdata,fitydata,wts,z0indexes,z0);

%Function for nomad
fun = @(theta_optim) cstr_optim(theta_optim,'data',gas_volume,temp,run(runs),run_lb,fitindexes,fitxdata,fitydata,wts,z0indexes,z0,minidx,rangeidx);



% nopts = nomadset('vns_search',1);
% opts = optiset('solver','nomad','display','iter','maxtime',120,'maxiter',50,'solverOpts',nopts); %

nopts = nloptset('algorithm','LN_COBYLA'); %LN_SBPLX

opts = optiset('solver','nlopt','display','iter','maxtime',28800,'maxiter',1e12,'solverOpts',nopts,'warnings','all'); %

%thetaOpt = opti_nomad(fun,theta,lb,ub,[],[],[],opts);
Opt = opti('fun',fun,'x0',theta,'bounds',lb,ub,'options',opts)

%Opt2 = opti('fun',fun_swap,'x0',flipud(theta),'bounds',flipud(lb),flipud(ub),'options',opts)


%%
[thetaOpt,f] = solve(Opt)

%%
save 'theta.mat' thetaOpt

%%
%ODE Function for Integrator
load 'theta_thesis.mat'
mixing = [run(runs).mixing]';
ode = @(t,z) cstr_calc(t,z,thetaOpt,'data',gas_volume,[run_lb(runs).temp],mixing);

z0e = thetaOpt(end);
zinit = z0;
zinit(z0indexes) = zinit(z0indexes)*z0e; %pCODfast
zinit(z0indexes+1) = zinit(z0indexes+1)*(1-z0e); %pCODslow

%Solve System
[t,y] = ode45(ode,[0 3600],zinit);

%%
cstr_vec_plot(run,t,y,1:19);
squeezefigs(3,3)

%%
%Plot Results to file
for i=1:(nz/states)
    %cstr_plot(run(i+1),t,y(:,((i*states)+1:(i*states)+states)));
    cstr_vec_plot(run,t,y,i);
    %squeezefigs(3,3)
    print(gcf,'-dpng',sprintf('cstrpic_%s_run%d',opts.solver,i));
end

%% Write results to res struct

for i=1:runs(end)
    res(i).t = t;
    res(i).pcod_f = y(:,(1+(i-1)*states));
    res(i).pcod_s = y(:,(2+(i-1)*states));
    res(i).pcod = y(:,(3+(i-1)*states));
    res(i).scod_f = y(:,(4+(i-1)*states));
    res(i).scod_s = y(:,(5+(i-1)*states));
    res(i).scod_nr = y(:,(6+(i-1)*states));
    res(i).scod = y(:,(7+(i-1)*states));
    res(i).ac = y(:,(8+(i-1)*states));
    res(i).co2 = y(:,(9+(i-1)*states));
    res(i).o2gas = y(:,(10+(i-1)*states));
    res(i).don = y(:,(11+(i-1)*states));
    res(i).vfacod_f= y(:,(12+(i-1)*states));
    res(i).vfacod_s = y(:,(13+(i-1)*states));
    res(i).vfacod = y(:,(14+(i-1)*states));
    
    delete(sprintf('test%i.xlsx',i));
    xlswrite(sprintf('test%i.xlsx',i), [res(i).t,  res(i).pcod_f, res(i).pcod_s, res(i).pcod, res(i).scod_f, res(i).scod_s, res(i).scod_nr, res(i).scod, res(i).ac, res(i).co2, res(i).o2gas, res(i).don, res(i).vfacod_f, res(i).vfacod_s, res(i).vfacod]);
    
    tm = run(1).time;
    pcod = interp1(t,res(i).pcod,tm);
    pcod_err = ((pcod-run(i).pcod)./run(i).pcod)*100;
    scod = interp1(t,res(i).scod,tm);
    scod_err = ((scod-(run(i).scod-run(i).vfa))./(run(i).scod-run(i).vfa))*100;
    ac = interp1(t,res(i).ac,tm);
    ac_err = ((ac-run(i).ac)./run(i).ac)*100;
    vfa = interp1(t,res(i).vfacod,tm);
    vfa_err = ((vfa-run(i).vfa-run(i).ac)./(run(i).vfa-run(i).ac))*100;
    don = interp1(t,res(i).don,tm);
    don_err = ((don-run(i).don)./run(i).don)*100;
    delete(sprintf('test%i_err.xlsx',i));
    xlswrite(sprintf('test%i_err.xlsx',i), {'Time', 'pCOD expt', 'pCOD', 'pCOD err', 'sCOD expt', 'sCOD', 'sCOD err', 'ac expt', 'ac', 'ac err', 'vfa expt', 'vfa', 'vfa err', 'don expt', 'don', 'don err'});
    xlswrite(sprintf('test%i_err.xlsx',i), [tm, run(i).pcod, pcod, pcod_err, run(i).scod, scod, scod_err, run(i).ac, ac, ac_err, run(i).vfa, vfa, vfa_err, run(i).don, don, don_err], 1, 'A2');
end

save 'myresults.mat', 'res';

%% Using model to compare reaction conditions for image analysis

load('theta_thesis.mat');
runs = 7; % 220 deg, mixing 500 rpm, O2 40 bar
mixing = [run(runs).mixing]';

temp = run_lb(runs).temp;

ode = @(t,z) cstr_calc(t,z,thetaOpt,'data',gas_volume,temp,mixing);

z0e = thetaOpt(end);
zinit = z0;
zinit(z0indexes) = zinit(z0indexes)*z0e; %pCODfast
zinit(z0indexes+1) = zinit(z0indexes+1)*(1-z0e); %pCODslow

%Solve System
[ti,yi] = ode45(ode,[0 3600],zinit);

%%
cstr_vec_plot(run,ti,yi,7);
%squeezefigs(3,3)

%% 
save('image220deg40bar.mat','ti','yi');


%% Generate simulation plots for end of Chapter 6

% Compute temp effect at 220, 240 and 260 deg
% based on Run 2
load('theta_thesis.mat');

% 220 deg
temp = run_lb(2).temp-20;
mixing = [run(2).mixing]';
ode = @(t,z) cstr_calc(t,z,thetaOpt,'data',gas_volume,temp,mixing);

z0e = thetaOpt(end);
zinit = z0;
zinit(z0indexes) = zinit(z0indexes)*z0e; %pCODfast
zinit(z0indexes+1) = zinit(z0indexes+1)*(1-z0e); %pCODslow

%Solve System
[ti,yi] = ode45(ode,[0 3600],zinit);
save('exp220deg20bar.mat','ti','yi');

% 240 deg
temp = run_lb(2).temp;
mixing = [run(2).mixing]';
ode = @(t,z) cstr_calc(t,z,thetaOpt,'data',gas_volume,temp,mixing);

z0e = thetaOpt(end);
zinit = z0;
zinit(z0indexes) = zinit(z0indexes)*z0e; %pCODfast
zinit(z0indexes+1) = zinit(z0indexes+1)*(1-z0e); %pCODslow

%Solve System
[ti,yi] = ode45(ode,[0 3600],zinit);
save('exp240deg20bar.mat','ti','yi');

% 260 deg
temp = run_lb(2).temp+20;
mixing = [run(2).mixing]';
ode = @(t,z) cstr_calc(t,z,thetaOpt,'data',gas_volume,temp,mixing);

z0e = thetaOpt(end);
zinit = z0;
zinit(z0indexes) = zinit(z0indexes)*z0e; %pCODfast
zinit(z0indexes+1) = zinit(z0indexes+1)*(1-z0e); %pCODslow

%Solve System
[ti,yi] = ode45(ode,[0 3600],zinit);
save('exp260deg20bar.mat','ti','yi');


%% Compute o2 effect at 20, 40 and 60 bar
% rerun o2 setup at top of file for each one
% 220 deg
% load('theta_thesis.mat');
% 
% temp = run_lb(2).temp-20;
% mixing = [run(2).mixing]';
% ode = @(t,z) cstr_calc(t,z,thetaOpt,'data',gas_volume,temp,mixing);
% 
% z0e = thetaOpt(end);
% zinit = z0;
% zinit(z0indexes) = zinit(z0indexes)*z0e; %pCODfast
% zinit(z0indexes+1) = zinit(z0indexes+1)*(1-z0e); %pCODslow
% 
% %Solve System
% [ti,yi] = ode45(ode,[0 3600],zinit);
% save('exp220deg20bar.mat','ti','yi');

%% 40 bar
load('theta_thesis.mat');
temp = run_lb(2).temp-20;
mixing = [run(2).mixing]';
ode = @(t,z) cstr_calc(t,z,thetaOpt,'data',gas_volume,temp,mixing);

z0e = thetaOpt(end);
zinit = z0;
zinit(z0indexes) = zinit(z0indexes)*z0e; %pCODfast
zinit(z0indexes+1) = zinit(z0indexes+1)*(1-z0e); %pCODslow

%Solve System
[ti,yi] = ode45(ode,[0 3600],zinit);
save('exp220deg40bar.mat','ti','yi');

%% 60 bar
load('theta_thesis.mat');
temp = run_lb(2).temp-20;
mixing = [run(2).mixing]';
ode = @(t,z) cstr_calc(t,z,thetaOpt,'data',gas_volume,temp,mixing);

z0e = thetaOpt(end);
zinit = z0;
zinit(z0indexes) = zinit(z0indexes)*z0e; %pCODfast
zinit(z0indexes+1) = zinit(z0indexes+1)*(1-z0e); %pCODslow

%Solve System
[ti,yi] = ode45(ode,[0 3600],zinit);
save('exp220deg60bar.mat','ti','yi');




