clc
clear all
format longG

%Load Scion Measurements
load SCION_DATA1;

pilotplant = 1;

% Constants
states = 14; %number of model states (used for indexing)
CO2 = 0; %Guesses Of Initial Concentration CO2
O2_mm = 31.9980; %O2 molar mass in g/mol
CO2_mm = 44.01; %CO2 molar mass in g/mol
H2O_mm = 18.01528; %H2O molar mass in g/mol


if pilotplant == 1
    ReactorVolume = 300; %(in l)
    
else
    ReactorVolume = 600; %Reactor volume in ml
    Water = 250; %Water added to reactor (ml)
end

fillTemperature = 293.15; %Fill temperature (K)
% Dissolved o2 calculation henry coefficients
kh = 0.0013; % mol/kg*bar
d = 1500; % 1/k

% Initialise variables
x = 1; % counter for indexing
fitydata = [];
xt = (0:10:3600)';
temp = cell(19,1);

%Runs of Interest
runs = 2:2;

for r=runs(1):runs(end)
    % Inital conditions
    
    if pilotplant == 0 % very different operating proceedure for pilot pland and parr reactor so different initial conditions setup required
        Sludge = run(r).sludge;
        TotLiquid = Water+Sludge;
        GasBeforeSludge = ReactorVolume-Water;
        GasAfterSludge = ReactorVolume-(Water+Sludge); %Gas vol after water and sludge
        vapPress = (Props('P','T',fillTemperature,'Q',0,'Water')/100)*1e5; %in Pa
        
        if run(r).noflush == 1
            % no oxygen purge so calculate amount of nitrogen in reactor
            n2Mole = ((101e3-vapPress)*(run(r).water/1e6))/(8.3145*fillTemperature); % reactor headspace flushed with pure nitrogen before oxygen addition
            o2Mole = (((run(r).o2*1e5)+101e3-vapPress)*(ReactorVolume-run(r).water)/1e6)/(8.3145*fillTemperature);
            %o2Mole = ((run(r).o2*1e5)*(ReactorVolume-run(r).water)/1e6)/(8.3145*(fillTemperature+273.15));
            o2Mole = o2Mole - n2Mole;
            
            fillPressure = (((o2Mole-n2Mole)*8.3145*fillTemperature)/((ReactorVolume-run(r).water)/1e6))/1e5;
        else
            %Initial Concentration of O2 with oxygen purge (no nitrogen in reactor)
            o2Mole = (((run(r).o2*1e5)+101e3-vapPress)*((ReactorVolume-run(r).water)/1e6))/(8.3145*fillTemperature); % calculate moles of O2 in gas using ideal gas law
            fillPressure = ((run(r).o2*1e5)+101e3)/1e5;
        end
        %Initial Concentration of O2 in gas phase at room temperature
        o2Mass = o2Mole*O2_mm; %mass of o2 in gas phase (g)
        % Calculate dissolved o2 at room temperature
        o2_dissolved = kh*exp(d*((1./fillTemperature)-(1/298))); % o2 dissolved in water (mol/kg*bar)
        o2_dissolvedmgL = o2_dissolved.*H2O_mm.*(fillPressure).*1000; % (mg/L)
        
        % Total oxygen in reactor
        massO2Liquid = o2_dissolvedmgL*Water/1e3; % (mg)
        totalO2InReactor = (massO2Liquid/1000)+o2Mass; %mass in (g)
        
        % Oxygen dissolved at operating temperature
        p = ((totalO2InReactor/O2_mm)*8.314*(run(r).temp(1)+273.15))/(GasAfterSludge/1e6)/1e5; % partial pressure after heating (bar)
        o2_dissolved_t0 = kh*exp(d*((1./(run(r).temp(1)+273.15))-(1/298))); % o2 dissolved in water (mol/kg*bar)
        o2_dissolvedmgL_t0 = o2_dissolved_t0.*H2O_mm.*(p).*1000; % (mg/L)
        
        % Total oxygen at operating temperature after sludge added
        massO2Liquid_t0 = o2_dissolvedmgL_t0*(TotLiquid/1e3); % (mg)
        totalO2InGas_t0 = (totalO2InReactor - massO2Liquid_t0/1000); %o2 in headspace (g)
        
        % Build reactor liquid level profile (due to sampling)
        gas_volume = ones(length(run_lb(r).pressure),1)*GasAfterSludge;
        gas_volume(12:30) = 220;
        gas_volume(30:60) = 240;
        gas_volume(60:120) = 280;
        gas_volume(120:end) = 300;
    else
        %Don't have any true cold start data for the pilot plant so use the
        %t=0 sample as the start
        
        TotLiquid = run(r).sludge(1)*ReactorVolume; % (ml) (l)
        
        vapPress = (Props('P','T',run_lb(r).temp(1),'Q',0,'Water')/100); %in Pa
        run(r).O2gasmass(1,1) = (run(r).O2gasfrac(1)*(run_lb(r).pressure(1)-vapPress)*1e5*(ReactorVolume - (run(r).sludge(1)*ReactorVolume))/1e3)/(8.3145*run_lb(r).temp(1))*O2_mm;  %(g)
       totalO2InReactor = run(r).O2gasmass(1,1);
       
       for j=1:length(run(r).O2gasfrac)
         TotLiquid = run(r).sludge(j)*ReactorVolume; % (ml)
        
        vapPress = (Props('P','T',run_lb(r).temp(j),'Q',0,'Water')/100); %in Pa
        run(r).O2gasmass(j,1) = (run(r).O2gasfrac(j)*(run_lb(r).pressure(j)-vapPress)*1e5*(ReactorVolume - (run(r).sludge(j)*ReactorVolume))/1e3)/(8.3145*run_lb(r).temp(j))*O2_mm;  %(g)          
       end

%     % Calculate dissolved o2 at room temperature
%     o2_dissolved = kh*exp(d*((1./fillTemperature)-(1/298))); % o2 dissolved in water (mol/kg*bar)
%     o2_dissolvedmgL = o2_dissolved.*H2O_mm.*(fillPressure).*1000; % (mg/L)
%     
%     % Total oxygen in reactor
%     massO2Liquid = o2_dissolvedmgL*Water/1e3; % (mg)
%     totalO2InReactor = (massO2Liquid/1000)+o2Mass; %mass in (g)
    
    % Oxygen dissolved at operating temperature
%     p = ((totalO2InReactor/O2_mm)*8.314*(run(r).temp(1)+273.15))/(GasAfterSludge/1e6)/1e5; % partial pressure after heating (bar)
    p = run(r).O2gasfrac(1)*(run_lb(r).pressure(1)-vapPress); % use o2 frac measurement and system pressure
    
    
    o2_dissolved_t0 = kh*exp(d*((1./(run_lb(r).temp(1)))-(1/298))); % o2 dissolved in water (mol/kg*bar)
    o2_dissolvedmgL_t0 = o2_dissolved_t0.*H2O_mm.*(p).*1000; % (mg/L)
    
    % Total oxygen at operating temperature after sludge added
    massO2Liquid_t0 = o2_dissolvedmgL_t0*(TotLiquid/1e3); % (mg)
    totalO2InGas_t0 = (totalO2InReactor - massO2Liquid_t0/1000); %o2 in headspace (g)
    
    % Build reactor liquid level profile (due to sampling)
    gas_volume = ReactorVolume*run(r).sludge; % Have reactor level measuremnt on the pilot plant
    
    end
    %liq_p = ((totalO2InReactor/O2_mm)*8.314*(run_lb(r).temp+273.15))./(gas_volume/1e6)/1e5;
    
    %Form Initial State Vector pCOD_f pCOD_s pCOD sCOD_f sCOD_s sCOD_nr sCOD AA nonAA VFA CO CO2 O2liq O2gas

   %totalO2InGas_t0 = totalO2InGas_t0/(ReactorVolume*run(r).sludge(1));
    
    z_prev(:,x) = [ run(r).pcod(1)*0.8
        run(r).pcod(1)*0.2
        run(r).pcod(1)
        (run(r).scod(1)-run(r).vfa(1)).*0.3
        (run(r).scod(1)-run(r).vfa(1))*0.5
        (run(r).scod(1)-run(r).vfa(1))*0.2
        run(r).scod(1)-run(r).vfa(1)
        run(r).ac(1)
        CO2
        totalO2InGas_t0*1000
        0 %run(r).don(1) %no don measurement for pilot plant
        (run(r).vfa(1)-run(r).ac(1))*0.7
        (run(r).vfa(1)-run(r).ac(1))*0.3
        run(r).vfa(1)-run(r).ac(1)];
    
    z(:,x) = [ run(r).pcod(1)
        run(r).pcod(1)
        run(r).pcod(1)
        (run(r).scod(1)-run(r).vfa(1)).*0.3
        (run(r).scod(1)-run(r).vfa(1))*0.5
        (run(r).scod(1)-run(r).vfa(1))*0.2
        run(r).scod(1)-run(r).vfa(1)
        run(r).ac(1)
        CO2
        totalO2InGas_t0*1000
        0 %run(r).don(1) %no don measurement for pilot plant
        (run(r).vfa(1)-run(r).ac(1))*0.7
        (run(r).vfa(1)-run(r).ac(1))*0.3
        run(r).vfa(1)-run(r).ac(1)];
    
    fitxdata(:,x) = run(r).time;
    
    if pilotplant == 0
    if run(r).O2gasfrac(end) > 0 % if run has gas data
        
        % prepend calculated inital conditions for o2 and co2 as we don't have
        % a t0 gas sample.
        run(r).CO2gasfrac(1) = 0;
        run(r).O2gasfrac(1) = 1;
        
        run(r).O2gasmass = run(r).O2gasfrac*totalO2InGas_t0;
        run(r).CO2gasmass = run(r).CO2gasfrac*totalO2InGas_t0*(CO2_mm/O2_mm);

        %fitydata = [fitydata [run(r).pcod run(r).scod-run(r).vfa run(r).ac run(r).O2gasfrac*totalO2InGas_t0*1000 run(r).don run(r).vfa-run(r).ac]];
        fitydata = [fitydata [run(r).pcod run(r).scod-run(r).vfa run(r).ac run(r).O2gasfrac*totalO2InGas_t0*1000 run(r).vfa-run(r).ac]];
        
    else
        run(r).O2gasmass = zeros(length(run(r).O2gasfrac),1);
        run(r).CO2gasmass = zeros(length(run(r).O2gasfrac),1);
        
        %fitydata = [fitydata [run(r).pcod run(r).scod-run(r).vfa run(r).ac run(r).don run(r).vfa-run(r).ac]];
        fitydata = [fitydata [run(r).pcod run(r).scod-run(r).vfa run(r).ac run(r).vfa-run(r).ac]];
    end
    else
        fitydata = [fitydata [run(r).pcod run(r).scod-run(r).vfa run(r).ac run(r).O2gasmass*1000 run(r).vfa-run(r).ac]];
    end
    
    x = x + 1;
end
z_prev = z_prev(:);
z0 = z(:);


%%
%Create parameter vector (theta)

ar=[1.8
    0.79485
    0.92597
    350.574026139142129 % 0.474026139142129
    30.2656022084079
    -1.70651
    0.594661484838351 % 0.694661484838351
    0.002526333
    0.0134430715091141
    0.869170145716102
    0.0114301957343009
    0.095928982356516]';

o2order = [         0.299605073861853
    0.886589177295454
    0.219100051097171
    0.925864750186 % 2.2900925864750186
    0.440205159262489
    0.248547314468428
    0.553828005481191
    0.126395513
    0.301627455525776
    0.364097328818647
    0.786130663591281
    0.40]';


arEa = [         0.401395704002056
    0.599175067950098
    0.35484705597372
    1.055676084595672 %0.855676084595672
    1.24094369530074
    0.472359392180145
    0.483266965035988
    1.788365906
    0.369764113444974
    0.345893785467609
    0.304417174544645
    0.43
    ]';



mixingorder = [         0.472071394186516
    0.545636358118566
    0.468192344739728
    1.2
    0.742152300876037
    0.455078638203035
    0.0534751244544175
    0.0238
    0.558189360812938
    0.0624943390960389
    0.001
    0.36
    ]';


theta = [ar o2order arEa mixingorder 0.5 0.3 0.5 0.84]; % added pCODf frac

% compute o2 flow profile
tdiff = [];
for a=2:length(run(runs).time)
        tdiff(a-1) = run(runs).time(a)-run(runs).time(a-1);
if a == length(run(runs).time)
    tdiff(a) = run(runs).time(a)-run(runs).time(a-1);
end
end
o2_perseconds = [];
o2_persecond = (run_lb(runs).o2flow./tdiff').*0.58; %0.6
o2_perseconds = o2_persecond(3:end);
o2_perseconds(end+1) = 0;
o2_perseconds(end+1) = 0;

%%
% compute reactor headspace o2 using reactor pressure and oxygen
% concentration (bypasses O2gas state)
for b=1:length(run(r).O2gasfrac)   
        TotLiquid = run(r).sludge(b)*ReactorVolume; % (ml) (l)
        vapPress = (Props('P','T',run_lb(r).temp(b),'Q',0,'Water')/100); %in Pa
        O2gas(b) = (run(r).O2gasfrac(b)*(run_lb(r).pressure(b)-vapPress)*1e5*(ReactorVolume - (run(r).sludge(b)*ReactorVolume))/1e3)/(8.3145*run_lb(r).temp(b))*O2_mm;  %(g)
end

% LOAD theata.mat!!!!!
load('theta.mat');
theta=thetaOpt;


%%
%ODE Function for Integrator
% ode = @(t,z) cstr_calc_pp(t,z,theta,'data',gas_volume,[run_lb(runs).temp],[run(runs).mixing]', run(runs).time, o2_perseconds);
ode = @(t,z) cstr_calc_pp(t,z,theta,'data',gas_volume,[run_lb(runs).temp],[run(runs).mixing]', run(runs).time, 1, O2gas.*1000);
tic
%Solve System
[t,y] = ode45(ode,[0 16200],z_prev); %16200
toc

%Run Simulation and Plot Results
cstr_vec_plot_pp(run,t,y,2);
%squeezefigs(3,3)

%%
cstr_vec_plot_pp(run,t,y,1);
squeezefigs(3,3)

%%
save('pilotplantresultsrun2.mat', 't','y');

%% overwrite plant data with computed o2 conc profile
save('SCION_DATA2.mat', 'run', 'run_lb');
%%
%Plot Results to file
for i=1:(nz/states)
    %cstr_plot(run(i+1),t,y(:,((i*states)+1:(i*states)+states)));
    cstr_vec_plot(run,t,y,i);
    %squeezefigs(3,3)
    print(gcf,'-dpng',sprintf('pilotplantpic_%s_run%d',opts.solver,i));
end

%% Write results to res struct

for i=1:runs(end)
    res(i).t = t;
    res(i).pcod_f = y(:,(1+(i-1)*states));
    res(i).pcod_s = y(:,(2+(i-1)*states));
    res(i).pcod = y(:,(3+(i-1)*states));
    res(i).scod_f = y(:,(4+(i-1)*states));
    res(i).scod_s = y(:,(5+(i-1)*states));
    res(i).scod_nr = y(:,(6+(i-1)*states));
    res(i).scod = y(:,(7+(i-1)*states));
    res(i).ac = y(:,(8+(i-1)*states));
    res(i).co2 = y(:,(9+(i-1)*states));
    res(i).o2gas = y(:,(10+(i-1)*states));
    res(i).don = y(:,(11+(i-1)*states));
    res(i).vfacod_f= y(:,(12+(i-1)*states));
    res(i).vfacod_s = y(:,(13+(i-1)*states));
    res(i).vfacod = y(:,(14+(i-1)*states));
    
    delete(sprintf('test%i.xlsx',i));
    xlswrite(sprintf('test%i.xlsx',i), [res(i).t,  res(i).pcod_f, res(i).pcod_s, res(i).pcod, res(i).scod_f, res(i).scod_s, res(i).scod_nr, res(i).scod, res(i).ac, res(i).co2, res(i).o2gas, res(i).don, res(i).vfacod_f, res(i).vfacod_s, res(i).vfacod]);
    
    tm = run(1).time;
    pcod = interp1(t,res(i).pcod,tm);
    pcod_err = ((pcod-run(i).pcod)./run(i).pcod)*100;
    scod = interp1(t,res(i).scod,tm);
    scod_err = ((scod-(run(i).scod-run(i).vfa))./(run(i).scod-run(i).vfa))*100;
    ac = interp1(t,res(i).ac,tm);
    ac_err = ((ac-run(i).ac)./run(i).ac)*100;
    vfa = interp1(t,res(i).vfacod,tm);
    vfa_err = ((vfa-run(i).vfa-run(i).ac)./(run(i).vfa-run(i).ac))*100;
    don = interp1(t,res(i).don,tm);
    don_err = ((don-run(i).don)./run(i).don)*100;
    delete(sprintf('test%i_err.xlsx',i));
    xlswrite(sprintf('test%i_err.xlsx',i), {'Time', 'pCOD expt', 'pCOD', 'pCOD err', 'sCOD expt', 'sCOD', 'sCOD err', 'ac expt', 'ac', 'ac err', 'vfa expt', 'vfa', 'vfa err', 'don expt', 'don', 'don err'});
    xlswrite(sprintf('test%i_err.xlsx',i), [tm, run(i).pcod, pcod, pcod_err, run(i).scod, scod, scod_err, run(i).ac, ac, ac_err, run(i).vfa, vfa, vfa_err, run(i).don, don, don_err], 1, 'A2');
end

save 'pilotplantresults.mat', 'res';
