from __future__ import division
import asim
import scipy as sp
import matplotlib.pyplot as plt
import time

"""
reaction = asim.reaction()
reaction.add("A -> B", [1], [1], 1, 1, 1, 1)
reaction.feedConc = {"A": 1, "B": 0};
reaction.feedTemp = 350;

reactor = asim.cstr(reaction, "reactor")
reactor.volume = 1
reactor.flowrate = 1
reactor.initial_conditions = {"A": 1, "B": 0};
reactor.initial_temp = 350
reactor.coolant_temp = 350
reactor.UA = 5e4

reactor2 = asim.cstr(reactor.output(), "reactor2")
reactor2.volume = 1
reactor2.flowrate = 1
reactor2.initial_conditions = {"A": 0, "B": 0};
reactor2.initial_temp = 350
reactor2.coolant_temp = 350
reactor2.UA = 5e4

out = reactor2.output()
# Simualtion time
tf = 10
# Perform the simulation
results = asim.sim(out,tf)

#Plot results
y = sp.arange(0,tf,0.01)

# Plot temperature
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)
ax1.set_title("Temperature of rectants vs time")
ax1.set_xlabel("Time (s)")
ax1.set_ylabel("Temperature (K)")
ax1.plot(y,results.UnitOps[results.Map["reactor"]].Temp)

# Plot concentration
fig2 = plt.figure()
ax2 = fig2.add_subplot(111)
ax2.set_title("Concentration of reaction compounds vs time")
ax2.set_xlabel("Time (s)")
ax2.set_ylabel("Concentration (mol/L)")
ax2.plot(y,results.UnitOps[0].Conc)
ax2.legend(('A', 'B'), 'upper right', shadow=True)

plt.show()

"""
#Hypothetical exothermic A->B reaction from Nonlinear Process Control book by Michael Henson and Dale Seborg

vss_molarmass = 22.665
vss_mgL = 12394.6666666667
vss_mol = ((vss_mgL/1000)*0.15)/vss_molarmass


# Create reaction object
reaction = asim.reaction()
# Configure elementary reaction
reaction.add("A -> B", [7.2e10], [1], 72751.88, 5e4, 0.239, 1000)
reaction.feedConc = {"A": 1.0, "B": 0}
reaction.feedTemp = 350

# Add a reactor
reactor = asim.cstr(reaction, "MainReactor")
# Specify reactor volume etc
reactor.volume = 100 # volume in L
reactor.flowrate = 100 # flow in L/min
reactor.initial_conditions = {"A": 0.5, "B": 0} # initial concentraions in mol/L
reactor.initial_temp = 350 # temperatures in K
reactor.coolant_temp = 300
reactor.UA = 5e4 # heat transfer area J/minK

out = reactor.output()

# Set simualtion time
tf = 10
# Perform the simulation
results = asim.sim(out,tf)

#Plot results
y = sp.arange(0,tf,0.01)

# Plot temperature
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)
ax1.set_title("Reactor Temperature")
ax1.set_xlabel("Time (s)")
ax1.set_ylabel("Temperature (K)")
ax1.plot(y,results.UnitOps[results.Map["MainReactor"]].Temp)

# Plot concentration
fig2 = plt.figure()
ax2 = fig2.add_subplot(111)
ax2.set_title("Concentration of Species vs Time")
ax2.set_xlabel("Time (s)")
ax2.set_ylabel("Concentration (mol/L)")
ax2.plot(y,results.UnitOps[results.Map["MainReactor"]].Conc)
ax2.legend(('A', 'B'), 'upper left', shadow=True)

plt.show()




"""
This example implements the lumped kinetic model from the paper
"Generalized Kinetic Model for Wet Oxidation of Organic Compounds" by Li, Chen and Gloyna
"""

"""
oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("Sludge + O2 -> End", k=[1.2e4], m=[1,1], E=167e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("Sludge + O2 -> CH3COOH", k=[3.04e3], m=[1,1], E=60e3, H=5e4, Cp=4.1855, rho=1000)
oxidation.add("CH3COOH + O2 -> End", k=[5.6e3], m=[1,0.37], E=67e3, H=5e4, Cp= 4.1855, rho=1000)
 
# Set initial conditions
oxidation.feedConc = {"Sludge": 1, "CH3COOH": 0, "End": 0, "O2": 2}
oxidation.feedTemp = 273

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation, "Main")
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 273
cstr1.coolant_temp = 573
cstr1.flowrate = 0
cstr1.initial_conditions =  {"Sludge": 1, "CH3COOH": 0, "End": 0, "O2": 2}
cstr1.UA = 5e4          # cstr heat transfer area

out = cstr1.output()

"""

"""
pfr = asim.plugflow(oxidation)
pfr.diameter = 0.05     # pfr pipe diameter
pfr.flowrate = 0.0001      # pfr flowrate
pfr.length = 50         # pfr length
pfr.initial_conditions = {"A": 1.0, "B": 0, "C": 0, "O": 2}
pfr.UA = 5e4
pfr._plugflow__n = 20
pfr.initial_temp = 493
pfr.coolant_temp = 493

pfr = asim.plugflow(oxidation)
pfr.diameter = 0.05     # pfr pipe diameter
pfr.flowrate = 0.0001      # pfr flowrate
pfr.length = 50         # pfr length
pfr.initial_conditions = {"A": 0, "B": 0, "C": 0, "O": 2}
pfr.UA = 5e4
pfr._plugflow__n = 20
pfr.initial_temp = 493
pfr.coolant_temp = 493


out2 = pfr.output()
"""
"""

vss_molarmass = 22.665
vss_mgL = 12394.6666666667
vss_mol = ((vss_mgL/1000)*0.15)/vss_molarmass

oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("VSS -> NH3 + VDS", k=[1.2e1], m=[0.1], E=100e2, H=5e4, Cp= 0.239, rho=1000)
#oxidation.add("VDS + O2 + H2O -> CH3COOH + NH3", k=[3.04e5], m=[1,1,1], E=60e3, H=5e4, Cp=4.1855, rho=1000)
#oxidation.add("VDS + O2 -> CO2 + NH3 + H2O", k=[5.6e6], m=[1,1], E=67e3, H=5e4, Cp= 4.1855, rho=1000)
 
# Set initial conditions
#oxidation.feedConc = {"VSS": 15000, "VDS": 0, "H2O": 0, "NH3": 0, "CH3COOH": 0, "CO2": 0, "O2": 22000}
oxidation.feedConc = {"VSS": 15000, "NH3": 0, "VDS":0} 
oxidation.feedTemp = 493

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation, "Main")
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 493
cstr1.coolant_temp = 493
cstr1.flowrate = 0
#cstr1.initial_conditions = {"VSS": 15000, "VDS": 0, "H2O": 0, "NH3": 0, "CH3COOH": 0, "CO2": 0, "O2": 22000}
cstr1.initial_conditions= {"VSS": 15000, "NH3": 0, "VDS":0} 
cstr1.UA = 5e4          # cstr heat transfer area

out = cstr1.output()


# Set simulation time
tf = 3600 # 1 hour

# Perform simulation
x = asim.sim(out, tf)



asim.gen_script(out,tf)

#kest = asim.regress(out, x, tf)

# Create x axis (time)
#interval = tf/len(x)
y = sp.arange(0,tf,0.01)

#conc_cstr = x[:,0:4]
#conc_pfr = x[:,-5:-1]

#total = x[:,0] + x[:,1]

# Plot temperature
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)
ax1.set_title("Temperature of rectants vs time")
ax1.set_xlabel("Time (s)")
ax1.set_ylabel("Temperature (K)")
ax1.plot(y,x.UnitOps[x.Map["Main"]].Temp)
#ax1.set_ylim([480, 500])

# Plot concentration
fig2 = plt.figure()
ax2 = fig2.add_subplot(111)
ax2.set_title("Concentration of reaction compounds vs time")
ax2.set_xlabel("Time (s)")
ax2.set_ylabel("Concentration (mol/L)")
ax2.plot(y,x.UnitOps[x.Map["Main"]].Conc)
#ax2.legend(('Prod X', 'O2', 'End prod', 'Acetic acid'), 'upper right', shadow=True)

plt.show()
"""