from __future__ import division
import asim
import scipy as sp
import matplotlib.pyplot as plt
import time

"""
oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("A -> B", [7.2e10], 5e4, 0.239, 1000)
# Set initial conditions
oxidation.feedConc = {"A": 1, "B": 0}
oxidation.feedTemp = 350

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation)
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 350
cstr1.coolant_temp = 305
cstr1.flowrate = 100
cstr1.initial_conditions = {"A": 0.5, "B": 0}
cstr1.UA = 5e4

out = cstr1.output()

# Set simulation time
tf = 10

# Perform simulation
x = asim.sim(out, tf)

# Extract concentration and temperature to seperate variables to make plotting easier
conc = x[:,0:2]
temp = x[:,-1]

# Create x axis (time)
interval = tf/len(x)
y = sp.arange(0,tf,interval)

# Plot temperature
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)
ax1.set_title("Temperature of rectants vs time")
ax1.set_xlabel("Time (s)")
ax1.set_ylabel("Temperature (K)")
ax1.plot(y,temp)

# Plot concentration
fig2 = plt.figure()
ax2 = fig2.add_subplot(111)
ax2.set_title("Concentration of reaction compounds vs time")
ax2.set_xlabel("Time (s)")
ax2.set_ylabel("Concentration (mol/L)")
ax2.plot(y,conc)
ax2.legend(('A', 'B'), 'upper right', shadow=True)

plt.show()


"""
oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("A -> C", [1.2e4], 67e3, 5e4, 0.239, 1000)
# Set initial conditions
oxidation.feedConc = {"A": 1, "B": 0}
oxidation.feedTemp = 350

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation)
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 350
cstr1.coolant_temp = 305
cstr1.flowrate = 100
cstr1.initial_conditions = {"A": 0.5, "B": 0}
cstr1.UA = 5e4          # cstr heat transfer area

# Create another reactor (pfr)

pfr = asim.plugflow(cstr1.output()) # Create pfr and link to the output of the cstr
pfr.coolant_temp = 305  # pfr coolant temp
pfr.diameter = 0.05     # pfr pipe diameter
pfr.flowrate = 100      # pfr flowrate
pfr.length = 10         # pfr length
pfr.initial_conditions = {"A": 0.5, "B": 0} # Initial compound concentrations in reactor
pfr.UA = 5e4            # pfr heat transfer area

out = pfr.output()

# Set simulation time
tf = 30

# Perform simulation
x = asim.sim(out, tf)

# Extract concentration and temperature to seperate variables to make plotting easier
conc_cstr = x[:,0:2]
conc_pfr = x[:,-3:-1]
temp_cstr = x[:,2]
temp_pfr = x[:,-1]

# Create x axis (time)
interval = tf/len(x)
y = sp.arange(0,tf,interval)

# Plot temperature
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)
ax1.set_title("Temperature of rectants vs time")
ax1.set_xlabel("Time (s)")
ax1.set_ylabel("Temperature (K)")
#ax1.plot(y,{temp_cstr,temp_pfr})

# Plot concentration of cstr
fig2 = plt.figure()
ax2 = fig2.add_subplot(111)
ax2.set_title("Concentration of reaction compounds vs time (CSTR)")
ax2.set_xlabel("Time (s)")
ax2.set_ylabel("Concentration (mol/L)")
ax2.plot(y,conc_cstr)
ax2.legend(('A', 'B'), 'upper right', shadow=True)

fig3 = plt.figure()
ax3 = fig3.add_subplot(111)
ax3.set_title("Concentration of reaction compounds vs time (PFR)")
ax3.set_xlabel("Time (s)")
ax3.set_ylabel("Concentration (mol/L)")
ax3.plot(y,conc_pfr)
ax3.legend(('A', 'B'), 'upper right', shadow=True)


plt.show()

print("test")