#!/usr/bin/python

import scipy as sp
import scipy.integrate as itg
import matplotlib as mpl
import matplotlib.pyplot as plt
import math
'''
## Reaction ##

A -> B


## Mapping ##

A	0	 -1*rxn_0(x[1], x[0], k, x[2]) + 1.0*(1 - x[0])
B	1	 +1*rxn_0(x[1], x[0], k, x[2]) + 1.0*(0 - x[1])
A	2	+rxn_0(x[1], x[0], k, x[2])*(1/(1*1)) + 1.0*(350 - x[2]) + (50000.0/(0.239*1000*1))*(350-x[2])
B	3	 -1*rxn_0(x[4], x[3], k, x[5]) + 1.0*(x[0] - x[3])
A	4	 +1*rxn_0(x[4], x[3], k, x[5]) + 1.0*(x[1] - x[4])
B	5	+rxn_0(x[4], x[3], k, x[5])*(1/(1*1)) + 1.0*(x[2] - x[5]) + (50000.0/(0.239*1000*1))*(350-x[5])
'''

def main():

    # Rate constants
    k = [1]

    # Kinetic rate equations
    #A -> B
    rxn_0 = lambda B, A, k, t : k[0]*math.exp(-1/(8.314*t))* A**1

    xdot = lambda x, t, k: sp.array([\
     -1*rxn_0(x[1], x[0], k, x[2]) + 1.0*(1 - x[0]),\
     +1*rxn_0(x[1], x[0], k, x[2]) + 1.0*(0 - x[1]),\
    +rxn_0(x[1], x[0], k, x[2])*(1/(1*1)) + 1.0*(350 - x[2]) + (50000.0/(0.239*1000*1))*(350-x[2]),\
     -1*rxn_0(x[4], x[3], k, x[5]) + 1.0*(x[0] - x[3]),\
     +1*rxn_0(x[4], x[3], k, x[5]) + 1.0*(x[1] - x[4]),\
    +rxn_0(x[4], x[3], k, x[5])*(1/(1*1)) + 1.0*(x[2] - x[5]) + (50000.0/(0.239*1000*1))*(350-x[5])\
    ])	

    #Initial concentrations:
    x0 = sp.array([\
    #A
    1,\
    #B
    0,\
    #A
    350,\
    #B
    0,\
    #A
    0,\
    #B
    350,\
    ])

    t = sp.arange(0, 10, 0.01)
    x = sp.integrate.odeint(xdot, x0, t, args=(k,))
    return x

if __name__ == '__main__':
    main()