from __future__ import division
import asim
import scipy as sp
import matplotlib.pyplot as plt
import time
import numpy as np

#Hypothetical exothermic A->B reaction from Nonlinear Process Control book by Michael Henson and Dale Seborg
#perform reaction at two different temperatures
"""
# Create reaction object
reaction = asim.reaction()
# Configure elementary reaction
reaction.add("A -> B", [7.2e10], [1], 72751.88, 5e4, 0.239, 1000)
reaction.feedConc = {"A": 1.0, "B": 0}
reaction.feedTemp = 350

# Add a reactor
reactor = asim.cstr(reaction, "MainReactor")
# Specify reactor volume etc
reactor.volume = 100 # volume in L
reactor.flowrate = 100 # flow in L/min
reactor.initial_conditions = {"A": 0.5, "B": 0} # initial concentraions in mol/L
reactor.initial_temp = 350 # temperatures in K
reactor.coolant_temp = 295
reactor.UA = 5e4 # heat transfer area J/minK

out = reactor.output()

# Set simualtion time
tf = 10
# Perform the simulation
results_before = asim.sim(out,tf)

# Create reaction object
reaction = asim.reaction()
# Configure elementary reaction
reaction.add("A -> B", [7.2e10], [1], 72751.88, 5e4, 0.239, 1000)
reaction.feedConc = {"A": 1.0, "B": 0}
reaction.feedTemp = 350

# Add a reactor
reactor = asim.cstr(reaction, "MainReactor")
# Specify reactor volume etc
reactor.volume = 100 # volume in L
reactor.flowrate = 100 # flow in L/min
reactor.initial_conditions = {"A": 0.5, "B": 0} # initial concentraions in mol/L
reactor.initial_temp = 350 # temperatures in K
reactor.coolant_temp = 305
reactor.UA = 5e4 # heat transfer area J/minK

out = reactor.output()

# Set simualtion time
tf = 10
# Perform the simulation
results_after = asim.sim(out,tf)

#Plot results
y = sp.arange(0,tf,0.01)



tdata = np.hstack((results_before.UnitOps[results_before.Map["MainReactor"]].Temp, results_after.UnitOps[results_after.Map["MainReactor"]].Temp));

# Plot temperature for 300K and 305K
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)
ax1.set_title("Reactor Internal Temperature at Differant Coolant Temperatures")
ax1.set_xlabel("Time (s)")
ax1.set_ylabel("Temperature (K)")
ax1.plot(y,tdata)
ax1.legend(('-5K', '+5K'), 'upper right')
ax1.set_ylim(310, 450)
leg = plt.gca().get_legend()
ltext = leg.get_texts()
plt.setp(ltext,fontsize='10')

# Plot concentration at 295K
fig2 = plt.figure()
ax2 = fig2.add_subplot(111)
ax2.set_title("Concentration of Species versus Time at 295K Coolant Temperature")
ax2.set_xlabel("Time (s)")
ax2.set_ylabel("Concentration (mol/L)")
ax2.plot(y,results_before.UnitOps[results_before.Map["MainReactor"]].Conc)
ax2.legend(('Compound A', 'Compound B'), 'best')
ax2.set_ylim(0, 1)
leg = plt.gca().get_legend()
ltext = leg.get_texts()
plt.setp(ltext,fontsize='10')

# Plot concentration at 305K
fig3 = plt.figure()
ax3 = fig3.add_subplot(111)
ax3.set_title("Concentration of Species versus Time at 305K Coolant Temperature")
ax3.set_xlabel("Time (s)")
ax3.set_ylabel("Concentration (mol/L)")
ax3.plot(y,results_after.UnitOps[results_after.Map["MainReactor"]].Conc)
ax3.legend(('Compound A', 'Compound B'), 'upper left')
ax3.set_ylim(0, 1)
leg = plt.gca().get_legend()
ltext = leg.get_texts()
plt.setp(ltext,fontsize='10')

plt.show()

"""
## Li et al triangular kinetic model
"""
This example implements the lumped kinetic model from the paper
"Generalized Kinetic Model for Wet Oxidation of Organic Compounds" by Li, Chen and Gloyna
"""

# Simulate 453K temp
oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("Sludge -> End", k=[1.2e4], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("Sludge -> CH3COOH", k=[1.2e4*0.15], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("CH3COOH -> End", k=[5.6e10], m=[1,0.37], E=167.7e3, H=5e4, Cp= 4.1855, rho=1000)

# Set initial conditions
oxidation.feedConc = {"Sludge": 100, "CH3COOH": 0, "End": 0}
oxidation.feedTemp = 453

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation, "Main")
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 453
cstr1.coolant_temp = 453
cstr1.flowrate = 0
cstr1.initial_conditions =  {"Sludge": 100, "CH3COOH": 0, "End": 0}
cstr1.UA = 5e4          # cstr heat transfer area

out = cstr1.output()

# Set simualtion time
tf = 3600
# Perform the simulation
results453 = asim.sim(out,tf)

# Simulate 473K temp

oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("Sludge -> End", k=[1.2e4], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("Sludge -> CH3COOH", k=[1.2e4*0.15], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("CH3COOH -> End", k=[5.6e10], m=[1,0.37], E=167.7e3, H=5e4, Cp= 4.1855, rho=1000)

# Set initial conditions
oxidation.feedConc = {"Sludge": 100, "CH3COOH": 0, "End": 0}
oxidation.feedTemp = 473

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation, "Main")
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 473
cstr1.coolant_temp = 473
cstr1.flowrate = 0
cstr1.initial_conditions =  {"Sludge": 100, "CH3COOH": 0, "End": 0}
cstr1.UA = 5e4          # cstr heat transfer area

out = cstr1.output()

# Set simualtion time
tf = 3600
# Perform the simulation
results473 = asim.sim(out,tf)

# Simulate 493K temp

oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("Sludge -> End", k=[1.2e4], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("Sludge -> CH3COOH", k=[1.2e4*0.15], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("CH3COOH -> End", k=[5.6e10], m=[1,0.37], E=167.7e3, H=5e4, Cp= 4.1855, rho=1000)

# Set initial conditions
oxidation.feedConc = {"Sludge": 100, "CH3COOH": 0, "End": 0}
oxidation.feedTemp = 493

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation, "Main")
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 493
cstr1.coolant_temp = 493
cstr1.flowrate = 0
cstr1.initial_conditions =  {"Sludge": 100, "CH3COOH": 0, "End": 0}
cstr1.UA = 5e4          # cstr heat transfer area

out = cstr1.output()

# Set simualtion time
tf = 3600
# Perform the simulation
results493 = asim.sim(out,tf)

# Simulate 513K temp

oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("Sludge -> End", k=[1.2e4], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("Sludge -> CH3COOH", k=[1.2e4*0.15], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("CH3COOH -> End", k=[5.6e10], m=[1,0.37], E=167.7e3, H=5e4, Cp= 4.1855, rho=1000)

# Set initial conditions
oxidation.feedConc = {"Sludge": 100, "CH3COOH": 0, "End": 0}
oxidation.feedTemp = 513

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation, "Main")
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 513
cstr1.coolant_temp = 513
cstr1.flowrate = 0
cstr1.initial_conditions =  {"Sludge": 100, "CH3COOH": 0, "End": 0}
cstr1.UA = 5e4          # cstr heat transfer area

out = cstr1.output()

# Set simualtion time
tf = 3600
# Perform the simulation
results513 = asim.sim(out,tf)

# Simulate 563K temp

oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("Sludge -> End", k=[1.2e4], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("Sludge -> CH3COOH", k=[1.2e4*0.15], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("CH3COOH -> End", k=[5.6e10], m=[1,0.37], E=167.7e3, H=5e4, Cp= 4.1855, rho=1000)

# Set initial conditions
oxidation.feedConc = {"Sludge": 100, "CH3COOH": 0, "End": 0}
oxidation.feedTemp = 563

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation, "Main")
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 563
cstr1.coolant_temp = 563
cstr1.flowrate = 0
cstr1.initial_conditions =  {"Sludge": 100, "CH3COOH": 0, "End": 0}
cstr1.UA = 5e4          # cstr heat transfer area

out = cstr1.output()

# Set simualtion time
tf = 3600
# Perform the simulation
results563 = asim.sim(out,tf)

#Plot results
y = sp.arange(0,tf,0.01)

# Plot temperature
"""
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)
ax1.set_title("Reactor Temperature")
ax1.set_xlabel("Time (s)")
ax1.set_ylabel("Temperature (K)")
ax1.plot(y,results.UnitOps[results.Map["Main"]].Temp)
"""

cod453 = results453.UnitOps[results453.Map["Main"]].Conc[:,0] + results453.UnitOps[results453.Map["Main"]].Conc[:,-1]
cod473 = results473.UnitOps[results473.Map["Main"]].Conc[:,0] + results473.UnitOps[results473.Map["Main"]].Conc[:,-1]
cod493 = results493.UnitOps[results493.Map["Main"]].Conc[:,0] + results493.UnitOps[results493.Map["Main"]].Conc[:,-1]
cod513 = results513.UnitOps[results513.Map["Main"]].Conc[:,0] + results513.UnitOps[results513.Map["Main"]].Conc[:,-1]
cod563 = results563.UnitOps[results563.Map["Main"]].Conc[:,0] + results563.UnitOps[results563.Map["Main"]].Conc[:,-1]

#tdata = np.hstack((results_before.UnitOps[results_before.Map["MainReactor"]].Temp, results_after.UnitOps[results_after.Map["MainReactor"]].Temp));

cod = np.hstack((cod453[:,np.newaxis],cod473[:,np.newaxis],cod493[:,np.newaxis],cod513[:,np.newaxis],cod563[:,np.newaxis]))

# Plot concentration
fig1 = plt.figure()
ax1 = fig1.add_subplot(111)
ax1.set_title("Python Modelling Environment using the Li et al Kinetic Model")
ax1.set_xlabel("Time (min)")
ax1.set_ylabel("Total COD Remaining (%)")
#ax2.plot(y,results.UnitOps[results.Map["Main"]].Conc)
ax1.set_ylim(0, 100)
ax1.set_xlim(0, 61)
ax1.plot(y/60,cod)
ax1.legend(('453K','473K','493K','513K','563K'), 'upper right')

plt.show()

# Our kinetic model version 1

oxidation = asim.reaction()

# Add a reaction A->B
oxidation.add("Sludge -> End", k=[1.2e4], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("Sludge -> CH3COOH", k=[1.2e4*0.15], m=[1,1], E=66.977e3, H=5e4, Cp= 4.1855, rho=1000)
oxidation.add("CH3COOH -> End", k=[5.6e10], m=[1,0.37], E=167.7e3, H=5e4, Cp= 4.1855, rho=1000)

# Set initial conditions
oxidation.feedConc = {"Sludge": 100, "CH3COOH": 0, "End": 0}
oxidation.feedTemp = 453

# Create a reactor (cstr)

cstr1 = asim.cstr(oxidation, "Main")
# Set initial conditions
cstr1.volume = 100
cstr1.initial_temp = 453
cstr1.coolant_temp = 453
cstr1.flowrate = 0
cstr1.initial_conditions =  {"Sludge": 100, "CH3COOH": 0, "End": 0}
cstr1.UA = 5e4          # cstr heat transfer area

out = cstr1.output()

# Set simualtion time
tf = 3600
# Perform the simulation
results453 = asim.sim(out,tf)
